from ursina import *
from random import uniform
from ursina.prefabs.first_person_controller import FirstPersonController


def input(key):
    global inventory_open
    if key == "escape":
        application.quit()

    if key == "i":
        if not inventory_open:
            inventory.enable()
            inventory_open = True
            mouse.locked = False  # Show cursor
        else:
            inventory.disable()
            inventory_open = False
            mouse.locked = True  # Hide cursor


def place_block():
    # Place a block two units in front of the player
    position = (
        round(player.position.x + camera.forward.x * 2),
        round(player.position.y + camera.forward.y * 2),
        round(player.position.z + camera.forward.z * 2)
    )
    block = Entity(
        model="cube",
        color=color.random_color(),
        texture="white_cube",
        position=position,
        scale=1,
        collider="box"
    )
    placed_blocks.append(block)


class Bee(Entity):
    """Bee entity that roams the map."""
    def __init__(self, **kwargs):
        super().__init__(
            model="Downloads/bee.glb",  # Path to your bee.glb file
            position=(uniform(-50, 50), 1, uniform(-50, 50)),  # Initial position
            scale=0.5,
            color=color.yellow,
            collider=None,
            **kwargs
        )
        self.speed = 2
        self.new_target()

    def update(self):
        # Move toward the target position
        self.position += (self.target - self.position).normalized() * self.speed * time.dt
        # Check if the bee is near the target, then pick a new one
        if distance(self.position, self.target) < 1:
            self.new_target()

    def new_target(self):
        """Set a new random target position within bounds."""
        self.target = Vec3(uniform(-500, 500), self.y, uniform(-500, 500))


# Ursina application
app = Ursina()

# Adding a custom skybox
Sky(texture="sky_sunset")  # Replace with your sky texture if available

# Ground setup
ground = Entity(
    model="plane",
    scale=(1000, 1, 1000),
    color=color.green,
    texture="grass",  # Replace with a realistic texture if available
    texture_scale=(500, 500),
    collider="box"
)

# Player setup
player = FirstPersonController(y=2, origin_y=-.5)

# Visible player hand
hand = Entity(
    model="cube",
    color=color.rgb(200, 150, 100),
    scale=(0.1, 0.2, 0.1),
    position=(player.x + 0.5, player.y - 0.5, player.z - 0.5),
    parent=player,
)

# Gun setup (attached to camera UI)
gun = Entity(
    model="assets/gun.obj",
    parent=camera.ui,
    scale=.08,
    color=color.gold,
    position=(.3, -.2),
    rotation=(-5, -10, -10)
)

# Walls setup
wall_1 = Entity(model="cube", collider="box", position=(-8, 0, 0), scale=(8, 5, 1), rotation=(0, 0, 0),
                texture="brick", texture_scale=(5, 5), color=color.rgb(255, 128, 0))
wall_2 = duplicate(wall_1, z=5)
wall_3 = duplicate(wall_1, z=10)
wall_4 = Entity(model="cube", collider="box", position=(-15, 0, 10), scale=(1, 5, 20), rotation=(0, 0, 0),
                texture="brick", texture_scale=(5, 5), color=color.rgb(255, 128, 0))

# Bee entity
bee = Bee()

# List to store placed blocks
placed_blocks = []

# Coordinates display
coordinates_text = Text(
    text="0, 0, 0",
    position=(-0.85, 0.45),  # Adjusted for better visibility
    origin=(0, 0),
    background=True
)

# Inventory setup
inventory_open = False
inventory = WindowPanel(
    title="Inventory",
    content=(
        Button("Place Block", color=color.azure, on_click=place_block),
    ),
    enabled=False,
    position=(0, 0)
)


# Update function
def update():
    # Update coordinates display
    coordinates_text.text = f"{int(player.x)}, {int(player.y)}, {int(player.z)}"

    # Gun and hand bobbing effect (only while walking)
    if held_keys['w'] or held_keys['a'] or held_keys['s'] or held_keys['d']:
        gun.position = (.3, -.2 + math.sin(time.time() * 6) * 0.02)
        hand.position = (
            player.x + 0.5 + math.sin(time.time() * 6) * 0.02,
            player.y - 0.5,
            player.z - 0.5
        )
    else:
        gun.position = (.3, -.2)
        hand.position = (player.x + 0.5, player.y - 0.5, player.z - 0.5)

    # Update bee
    bee.update()


# Run the app
app.run()
